@@ -14,7 +14,9 @@
 #include <algorithm> // for std::count()
 #include <cassert>
 #include <cctype> // for isdigit()
+#include <cmath>  // for floor()
 #include <iostream>
+#include <vector>
 
 namespace Avogadro::Core {
 
@@ -279,16 +281,15 @@ void SpaceGroups::fillUnitCell(Molecule& mol, unsigned short hallNumber,
       if (wrapToCell)
         newCandidate = uc->wrapCartesian(newCandidate);
 
-      // If there is already an atom in this location within a
+      // If there is already any atom in this location within a
       // certain tolerance, do not add the atom.
       bool atomAlreadyPresent = false;
       for (Index k = 0; k < mol.atomCount(); k++) {
-        // If it does not have the same atomic number, skip over it.
-        if (mol.atomicNumber(k) != atomicNum)
-          continue;
         Real distance = uc->distance(mol.atomPosition3d(k), newCandidate);
-        if (distance <= cartTol)
+        if (distance <= cartTol) {
           atomAlreadyPresent = true;
+          break; // no need to keep looking
+        }
       }
 
       // If there is already an atom present here, just continue
@@ -358,10 +359,6 @@ void SpaceGroups::fillUnitCell(Molecule& mol, unsigned short hallNumber,
       // certain tolerance, do not add the atom.
       bool atomAlreadyPresent = false;
       for (Index k = 0; k < mol.atomCount(); k++) {
-        // If it does not have the same atomic number, skip over it.
-        if (mol.atomicNumber(k) != atomicNum)
-          continue;
-
         Real distance = (mol.atomPosition3d(k) - newCandidate).norm();
 
         if (distance <= cartTol) {
@@ -424,6 +421,56 @@ void SpaceGroups::reduceToAsymmetricUnit(Molecule& mol,
   }
 }
 
+Array<Index> SpaceGroups::translationalUniqueAtoms(const Molecule& mol,
+                                                   double tolerance)
+{
+  Array<Index> uniqueIndices;
+
+  if (!mol.unitCell())
+    return uniqueIndices;
+
+  const UnitCell* uc = mol.unitCell();
+  Index numAtoms = mol.atomCount();
+
+  // Store wrapped coordinates for atoms we've already accepted
+  std::vector<std::pair<unsigned char, Vector3>> accepted;
+
+  for (Index i = 0; i < numAtoms; ++i) {
+    unsigned char atomicNum = mol.atomicNumber(i);
+    Vector3 fracCoords = uc->toFractional(mol.atomPosition3d(i));
+
+    // Wrap coordinates to [0, 1) range for comparison
+    Vector3 wrappedCoords;
+    for (int c = 0; c < 3; ++c) {
+      double coord = fracCoords[c];
+      coord = coord - floor(coord);
+      if (coord > 1.0 - tolerance)
+        coord = 0.0;
+      wrappedCoords[c] = coord;
+    }
+
+    // Check if this is a duplicate of a previously accepted atom
+    bool isDuplicate = false;
+    for (const auto& prev : accepted) {
+      if (prev.first != atomicNum)
+        continue;
+
+      // Check if positions match within tolerance
+      if ((wrappedCoords - prev.second).norm() < tolerance) {
+        isDuplicate = true;
+        break;
+      }
+    }
+
+    if (!isDuplicate) {
+      uniqueIndices.push_back(i);
+      accepted.emplace_back(atomicNum, wrappedCoords);
+    }
+  }
+
+  return uniqueIndices;
+}
+
 const char* SpaceGroups::transformsString(unsigned short hallNumber)
 {
   if (hallNumber <= 530)
