@@ -18,8 +18,10 @@
 
 #include <nlohmann/json.hpp>
 
+#include <cstdio>
 #include <iomanip>
 #include <iostream>
+#include <iterator>
 
 using json = nlohmann::json;
 
@@ -86,6 +88,57 @@ json eigenColToJson(const MatrixX& matrix, int column)
   return j;
 }
 
+// Sanitize a raw string by replacing invalid UTF-8 sequences with '?'
+// (to work around a bug in Open Babel space group output)
+std::string sanitizeUtf8(const std::string& s)
+{
+  std::string result;
+  result.reserve(s.size());
+  const unsigned char* bytes = reinterpret_cast<const unsigned char*>(s.data());
+  size_t len = s.size();
+
+  for (size_t i = 0; i < len; ++i) {
+    if (bytes[i] <= 0x7F) {
+      result.push_back(static_cast<char>(bytes[i]));
+      continue;
+    }
+
+    size_t remaining = 0;
+    if ((bytes[i] & 0xE0) == 0xC0)
+      remaining = 1;
+    else if ((bytes[i] & 0xF0) == 0xE0)
+      remaining = 2;
+    else if ((bytes[i] & 0xF8) == 0xF0)
+      remaining = 3;
+    else {
+      result.push_back('?'); // invalid lead byte
+      continue;
+    }
+
+    if (i + remaining >= len) {
+      result.push_back('?'); // truncated sequence
+      continue;
+    }
+
+    bool valid = true;
+    for (size_t j = 1; j <= remaining; ++j) {
+      if ((bytes[i + j] & 0xC0) != 0x80) {
+        valid = false;
+        break;
+      }
+    }
+
+    if (valid) {
+      for (size_t j = 0; j <= remaining; ++j)
+        result.push_back(static_cast<char>(bytes[i + j]));
+      i += remaining;
+    } else {
+      result.push_back('?'); // invalid continuation
+    }
+  }
+  return result;
+}
+
 bool CjsonFormat::read(std::istream& file, Molecule& molecule)
 {
   return deserialize(file, molecule, true);
@@ -95,6 +148,7 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
                               bool isJson)
 {
   json jsonRoot;
+
   // could throw parse errors
   try {
     if (isJson)
@@ -109,8 +163,18 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
     return false;
   }
 
+  if (jsonRoot.is_discarded() && isJson) {
+    // Initial parse failed - try sanitizing UTF-8 and re-parsing
+    file.clear();
+    file.seekg(0);
+    std::string content((std::istreambuf_iterator<char>(file)),
+                        std::istreambuf_iterator<char>());
+    std::string sanitizedContent = sanitizeUtf8(content);
+    jsonRoot = json::parse(sanitizedContent, nullptr, false);
+  }
+
   if (jsonRoot.is_discarded()) {
-    appendError("Error reading CJSON file.");
+    appendError("Error reading CJSON file. Root is discarded.");
     return false;
   }
 
@@ -279,15 +343,36 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
 
   if (atoms.contains("frozen")) {
     json frozen = atoms["frozen"];
-    if (isBooleanArray(frozen) && frozen.size() == atomCount) {
-      for (Index i = 0; i < atomCount; ++i)
-        molecule.setFrozenAtom(i, frozen[i]);
-    } // might also be a 3xN array for per-axis freezing
-    else if (isNumericArray(frozen) && frozen.size() == 3 * atomCount) {
+    if (frozen.is_array() && frozen.size() == atomCount) {
       for (Index i = 0; i < atomCount; ++i) {
-        molecule.setFrozenAtomAxis(i, 0, frozen[3 * i] != 0);
-        molecule.setFrozenAtomAxis(i, 1, frozen[3 * i + 1] != 0);
-        molecule.setFrozenAtomAxis(i, 2, frozen[3 * i + 2] != 0);
+        bool freeze = false;
+        if (frozen[i].is_number())
+          freeze = (frozen[i] != 0);
+        else if (frozen[i].is_boolean())
+          freeze = frozen[i];
+        molecule.setFrozenAtom(i, freeze);
+      }
+    } // might also be a 3xN array for per-axis freezing
+    else if (frozen.is_array() && frozen.size() == 3 * atomCount) {
+      for (Index i = 0; i < atomCount; ++i) {
+        bool freeze = false;
+        if (frozen[3 * i].is_number())
+          freeze = (frozen[3 * i] != 0);
+        else if (frozen[3 * i].is_boolean())
+          freeze = frozen[3 * i];
+        molecule.setFrozenAtomAxis(i, 0, freeze);
+
+        if (frozen[3 * i + 1].is_number())
+          freeze = (frozen[3 * i + 1] != 0);
+        else if (frozen[3 * i + 1].is_boolean())
+          freeze = frozen[3 * i + 1];
+        molecule.setFrozenAtomAxis(i, 1, freeze);
+
+        if (frozen[3 * i + 2].is_number())
+          freeze = (frozen[3 * i + 2] != 0);
+        else if (frozen[3 * i + 2].is_boolean())
+          freeze = frozen[3 * i + 2];
+        molecule.setFrozenAtomAxis(i, 2, freeze);
       }
     }
   }
@@ -305,6 +390,15 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
     }
   }
 
+  // look for isotopes if present
+  if (atoms.contains("isotopes")) {
+    json isotopes = atoms["isotopes"];
+    if (isNumericArray(isotopes) && isotopes.size() == atomCount) {
+      for (Index i = 0; i < atomCount; ++i)
+        molecule.setIsotope(i, isotopes[i]);
+    }
+  }
+
   // Bonds are optional, but if present should be loaded.
   if (jsonRoot.contains("bonds")) {
     json bonds = jsonRoot["bonds"];
@@ -323,7 +417,13 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
         if (isNumericArray(order)) {
           for (unsigned int i = 0; i < molecule.bondCount() && i < order.size();
                ++i) {
-            molecule.bond(i).setOrder(static_cast<int>(order[i]));
+            int bondOrder = static_cast<int>(order[i]);
+            if (bondOrder < 1 || bondOrder > 6) {
+              appendError("Error: bond order is invalid.");
+              return false;
+            }
+
+            molecule.bond(i).setOrder(bondOrder);
           }
         }
       }
@@ -391,6 +491,10 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
         }
 
         molecule.addResidue(newResidue);
+
+        json label = residue["label"];
+        if (label.is_string())
+          molecule.setResidueLabel(molecule.residueCount() - 1, label);
       }
     }
   }
@@ -434,18 +538,20 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
           return false;
         }
       }
-      if (unitCellObject != nullptr)
+      if (unitCellObject != nullptr) {
         molecule.setUnitCell(unitCellObject);
 
-      // check for Hall number if present
-      if (unitCell["hallNumber"].is_number()) {
-        auto hallNumber = static_cast<int>(unitCell["hallNumber"]);
-        if (hallNumber > 0 && hallNumber < 531)
-          molecule.setHallNumber(hallNumber);
-      } else if (unitCell["spaceGroup"].is_string()) {
-        auto hallNumber = Core::SpaceGroups::hallNumber(unitCell["spaceGroup"]);
-        if (hallNumber != 0)
-          molecule.setHallNumber(hallNumber);
+        // check for Hall number if present
+        if (unitCell["hallNumber"].is_number()) {
+          auto hallNumber = static_cast<int>(unitCell["hallNumber"]);
+          if (hallNumber > 0 && hallNumber < 531)
+            molecule.setHallNumber(hallNumber);
+        } else if (unitCell["spaceGroup"].is_string()) {
+          auto hallNumber =
+            Core::SpaceGroups::hallNumber(unitCell["spaceGroup"]);
+          if (hallNumber != 0)
+            molecule.setHallNumber(hallNumber);
+        }
       }
     }
   }
@@ -854,18 +960,77 @@ bool CjsonFormat::deserialize(std::istream& file, Molecule& molecule,
   }
 
   // Partial charges are optional, but if present should be loaded.
+  json partialCharges;
   if (atoms.contains("partialCharges")) {
-    json partialCharges = atoms["partialCharges"];
-    if (partialCharges.is_object()) {
-      // keys are types, values are arrays of charges
-      for (auto& kv : partialCharges.items()) {
-        MatrixX charges(atomCount, 1);
-        if (isNumericArray(kv.value()) && kv.value().size() == atomCount) {
-          for (size_t i = 0; i < kv.value().size(); ++i) {
-            charges(i, 0) = kv.value()[i];
-          }
-          molecule.setPartialCharges(kv.key(), charges);
+    partialCharges = atoms["partialCharges"];
+  }
+  // some inconsistent files have it as part of the root
+  if (jsonRoot.contains("partialCharges")) {
+    partialCharges = jsonRoot["partialCharges"];
+  }
+  if (partialCharges.is_object()) {
+    // keys are types, values are arrays of charges
+    for (auto& kv : partialCharges.items()) {
+      MatrixX charges(atomCount, 1);
+      if (isNumericArray(kv.value()) && kv.value().size() == atomCount) {
+        for (size_t i = 0; i < kv.value().size(); ++i) {
+          charges(i, 0) = kv.value()[i];
         }
+        molecule.setPartialCharges(kv.key(), charges);
+      }
+    }
+  }
+
+  // look for possible cube data
+  if (jsonRoot.find("cube") != jsonRoot.end()) {
+    json cubeObj = jsonRoot["cube"];
+    // get the limits
+    Vector3 min, max, delta;
+    Vector3i points;
+    json origin = cubeObj["origin"];
+    json spacing = cubeObj["spacing"];
+    json dimensions = cubeObj["dimensions"];
+    json type = cubeObj["type"];
+
+    if (isNumericArray(origin) && origin.size() == 3 &&
+        isNumericArray(spacing) && spacing.size() == 3 &&
+        isNumericArray(dimensions) && dimensions.size() == 3) {
+      Cube* cube = molecule.addCube();
+
+      // types
+      if (type == "vdw")
+        cube->setCubeType(Cube::VdW);
+      else if (type == "solventAccessible")
+        cube->setCubeType(Cube::SolventAccessible);
+      else if (type == "solventExcluded")
+        cube->setCubeType(Cube::SolventExcluded);
+      else if (type == "esp")
+        cube->setCubeType(Cube::ESP);
+      else if (type == "electronDensity")
+        cube->setCubeType(Cube::ElectronDensity);
+      else if (type == "spinDensity")
+        cube->setCubeType(Cube::SpinDensity);
+      else if (type == "mo")
+        cube->setCubeType(Cube::MO);
+      else
+        cube->setCubeType(Cube::FromFile);
+
+      if (cubeObj.find("name") != cubeObj.end())
+        cube->setName(cubeObj["name"]);
+
+      min = Vector3(origin[0], origin[1], origin[2]);
+      points = Vector3i(dimensions[0], dimensions[1], dimensions[2]);
+      delta = Vector3(spacing[0], spacing[1], spacing[2]);
+      max = Vector3(min[0] + (points[0] - 1) * delta[0],
+                    min[1] + (points[1] - 1) * delta[1],
+                    min[2] + (points[2] - 1) * delta[2]);
+
+      cube->setLimits(min, max, points);
+      // check the length of the scalar array
+      unsigned int expectedSize = points[0] * points[1] * points[2];
+      if (isNumericArray(cubeObj["scalars"]) &&
+          cubeObj["scalars"].size() == expectedSize) {
+        cube->setData(cubeObj["scalars"]);
       }
     }
   }
@@ -1192,13 +1357,9 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
 
   // Write out any cubes that are present in the molecule.
   if (molecule.cubeCount() > 0) {
-    const Cube* cube = molecule.cube(0);
-    json cubeData;
-    for (float it : *cube->data()) {
-      cubeData.push_back(it);
-    }
-    // Get the origin, max, spacing, and dimensions to place in the object.
     json cubeObj;
+    const Cube* cube = molecule.cube(0);
+    // Get the origin, max, spacing, and dimensions to place in the object.
     json cubeMin;
     cubeMin.push_back(cube->min().x());
     cubeMin.push_back(cube->min().y());
@@ -1214,6 +1375,43 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
     cubeDims.push_back(cube->dimensions().y());
     cubeDims.push_back(cube->dimensions().z());
     cubeObj["dimensions"] = cubeDims;
+
+    // type
+    switch (cube->cubeType()) {
+      case Cube::VdW:
+        cubeObj["type"] = "vdw";
+        break;
+      case Cube::SolventAccessible:
+        cubeObj["type"] = "solventAccessible";
+        break;
+      case Cube::SolventExcluded:
+        cubeObj["type"] = "solventExcluded";
+        break;
+      case Cube::ESP:
+        cubeObj["type"] = "esp";
+        break;
+      case Cube::ElectronDensity:
+        cubeObj["type"] = "electronDensity";
+        break;
+      case Cube::SpinDensity:
+        cubeObj["type"] = "spinDensity";
+        break;
+      case Cube::MO:
+        cubeObj["type"] = "mo";
+        break;
+      case Cube::FromFile:
+      default:
+        cubeObj["type"] = "fromFile";
+        break;
+    }
+
+    // name
+    cubeObj["name"] = cube->name();
+
+    json cubeData;
+    for (float it : *cube->data()) {
+      cubeData.push_back(it);
+    }
     cubeObj["scalars"] = cubeData;
     root["cube"] = cubeObj;
   }
@@ -1270,16 +1468,15 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
     }
 
     if (anyFrozen) {
-      if (axisFrozen) {
-        // iterate through the mask as an array
-        json frozenAtomMaskArray;
-        for (Index i = 0; i < frozenAtomMask.size(); ++i) {
-          frozenAtomMaskArray.push_back(frozenAtomMask[i]);
-        }
-        atoms["frozen"] = frozenAtomMaskArray;
-      } else {
-        atoms["frozen"] = frozen;
+      atoms["frozen"] = frozen;
+    }
+    if (axisFrozen) {
+      // iterate through the mask as an array
+      json frozenAtomMaskArray;
+      for (Index i = 0; i < frozenAtomMask.size(); ++i) {
+        frozenAtomMaskArray.push_back(frozenAtomMask[i]);
       }
+      atoms["frozen"] = frozenAtomMaskArray;
     }
 
     // check for partial charges
@@ -1383,10 +1580,21 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
     if (hasFormalCharges)
       atoms["formalCharges"] = formalCharges;
 
+    // isotopes (if present)
+    json isotopes;
+    const auto isotopeList = molecule.isotopes();
+    if (isotopeList.size() == molecule.atomCount()) {
+      for (Index i = 0; i < molecule.atomCount(); ++i) {
+        isotopes.push_back(isotopeList[i]);
+      }
+      atoms["isotopes"] = isotopes;
+    }
+
     auto layer = LayerManager::getMoleculeInfo(&molecule)->layer;
-    if (layer.atomCount()) {
+    if (layer.atomCount() && molecule.atomCount()) {
       json atomLayer;
-      for (Index i = 0; i < layer.atomCount(); ++i) {
+      // Use molecule atom count to avoid writing stale layer data
+      for (Index i = 0; i < molecule.atomCount(); ++i) {
         atomLayer.push_back(layer.getLayerID(i));
       }
       atoms["layer"] = atomLayer;
@@ -1427,6 +1635,7 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
   // Create and populate any residue arrays
   if (molecule.residues().size() > 0) {
     json residues; // array of objects
+    Array residueLabels = molecule.residueLabels();
     for (auto residue : molecule.residues()) {
       json entry;
       entry["name"] = residue.residueName();
@@ -1449,6 +1658,11 @@ bool CjsonFormat::serialize(std::ostream& file, const Molecule& molecule,
       }
       entry["atoms"] = atoms;
       residues.push_back(entry);
+
+      // do we have custom residue labels?
+      if (residueLabels.size() > residue.residueId() + 1) {
+        entry["label"] = residueLabels[residue.residueId()];
+      }
     }
     root["residues"] = residues;
 
