@@ -43,8 +43,10 @@ public:
   std::vector<std::string> fileExtensions() const override;
   std::vector<std::string> mimeTypes() const override;
 
-  bool read(std::istream& inStream, Core::Molecule& molecule) override;
-  bool write(std::ostream& outStream, const Core::Molecule& molecule) override;
+  [[nodiscard]] bool read(std::istream& inStream,
+                          Core::Molecule& molecule) override;
+  [[nodiscard]] bool write(std::ostream& outStream,
+                           const Core::Molecule& molecule) override;
 };
 
 } // namespace Avogadro::Io
