@@ -10,9 +10,10 @@
 namespace Avogadro::QtGui {
 
 BackgroundFileFormat::BackgroundFileFormat(Io::FileFormat* format,
-                                           QObject* aparent)
-  : QObject(aparent), m_format(format), m_molecule(nullptr), m_success(false)
-{}
+                                           QObject* parent)
+  : QObject(parent), m_format(format), m_molecule(nullptr), m_success(false)
+{
+}
 
 BackgroundFileFormat::~BackgroundFileFormat()
 {
@@ -69,4 +70,4 @@ void BackgroundFileFormat::write()
   emit finished();
 }
 
-} // namespace Avogadro
+} // namespace Avogadro::QtGui
