@@ -205,8 +205,8 @@ map<size_t, AtomsPairList> Cartoons::getBackboneByResidues(
   map<size_t, BackboneResidue> previousAtom;
   for (const auto& residue : molecule.residues()) {
     if (!residue.isHeterogen()) {
-      Atom caAtom = residue.getAtomByName("CA");
-      Atom oAtom = residue.getAtomByName("O");
+      Atom caAtom = residue.atomByName("CA");
+      Atom oAtom = residue.atomByName("O");
       if (caAtom.isValid() && oAtom.isValid() &&
           m_layerManager.atomEnabled(layer, caAtom.index()) &&
           m_layerManager.atomEnabled(layer, oAtom.index())) {
@@ -215,8 +215,8 @@ map<size_t, AtomsPairList> Cartoons::getBackboneByResidues(
         addBackBone(result, previousAtom, caAtom, residue.color(), group,
                     residue.secondaryStructure());
       } else { // maybe DNA
-        Atom c3Atom = residue.getAtomByName("C3'");
-        Atom o3Atom = residue.getAtomByName("O3'");
+        Atom c3Atom = residue.atomByName("C3'");
+        Atom o3Atom = residue.atomByName("O3'");
         if (c3Atom.isValid() && o3Atom.isValid() &&
             m_layerManager.atomEnabled(layer, c3Atom.index()) &&
             m_layerManager.atomEnabled(layer, o3Atom.index())) {
