@@ -12,6 +12,7 @@
 // for partial charges
 #include <avogadro/calc/chargemanager.h>
 
+#include <avogadro/core/contrastcolor.h>
 #include <avogadro/core/elements.h>
 #include <avogadro/core/residue.h>
 #include <avogadro/qtgui/colorbutton.h>
@@ -33,6 +34,7 @@ namespace Avogadro::QtPlugins {
 using Avogadro::Rendering::TextLabel3D;
 using Core::Array;
 using Core::Atom;
+using Core::contrastColor;
 using Core::Elements;
 using Core::Molecule;
 using QtGui::PluginLayerManager;
@@ -173,7 +175,7 @@ struct LayerLabel : Core::LayerData
       atom->setObjectName("atom");
 
       // set up the various atom options
-      char val = LabelOptions::None;
+      [[maybe_unused]] char val = LabelOptions::None;
       QStringList text;
 
       // first add the individual options
@@ -218,29 +220,14 @@ struct LayerLabel : Core::LayerData
 
       auto* residue = new QComboBox;
       residue->setObjectName("residue");
-      for (char i = 0x00; i < std::pow(2, 2); ++i) {
-        if (i == 0) {
-          residue->addItem(QObject::tr("None"), QVariant(LabelOptions::None));
-        } else {
-          char val = 0x00;
-          QStringList text;
-          if (i & LabelOptions::Index) {
-            text << QObject::tr("ID");
-            val |= LabelOptions::Index;
-          }
-          if (i & LabelOptions::Name) {
-            text << QObject::tr("Name");
-            val |= LabelOptions::Name;
-          }
-          if (val != 0x00) {
-            QString join = QObject::tr(" & ");
-            residue->addItem(text.join(join), QVariant(val));
-            if (val == residueOptions) {
-              residue->setCurrentText(text.join(join));
-            }
-          }
-        }
-      }
+
+      // set up the various residue options
+      residue->addItem(QObject::tr("None"), int(LabelOptions::None));
+      residue->addItem(QObject::tr("ID"), int(LabelOptions::Index));
+      residue->addItem(QObject::tr("Name"), int(LabelOptions::Name));
+      residue->addItem(QObject::tr("Name & ID"),
+                       int(LabelOptions::Index + LabelOptions::Name));
+      residue->addItem(QObject::tr("Custom"), int(LabelOptions::Custom));
       QObject::connect(residue, SIGNAL(currentIndexChanged(int)), slot,
                        SLOT(residueLabelType(int)));
 
@@ -285,13 +272,14 @@ void Label::processResidue(const Core::Molecule& molecule,
   node.addChild(geometry);
 
   for (const auto& residue : molecule.residues()) {
-    Atom caAtom = residue.getAtomByName("CA");
+    Atom caAtom = residue.atomByName("CA");
     if (!caAtom.isValid() ||
         !m_layerManager.atomEnabled(layer, caAtom.index())) {
       continue;
     }
     auto name = residue.residueName();
     const auto atoms = residue.residueAtoms();
+    const std::string customLabel = molecule.residueLabel(residue.residueId());
     Vector3f pos = Vector3f::Zero();
     for (const auto& atom : atoms) {
       pos += atom.position3d().cast<float>();
@@ -311,11 +299,14 @@ void Label::processResidue(const Core::Molecule& molecule,
     auto* interface = m_layerManager.getSetting<LayerLabel>(layer);
     Vector3ub color = interface->color;
     std::string text = "";
-    if (interface->residueOptions & LayerLabel::LabelOptions::Index) {
-      text = std::to_string(residue.residueId());
-    }
     if (interface->residueOptions & LayerLabel::LabelOptions::Name) {
-      text += (text == "" ? "" : " / ") + name;
+      text = (text == "" ? "" : " / ") + name;
+    }
+    if (interface->residueOptions & LayerLabel::LabelOptions::Index) {
+      text += std::to_string(residue.residueId());
+    }
+    if (interface->residueOptions & LayerLabel::LabelOptions::Custom) {
+      text += (text == "" ? "" : " / ") + customLabel;
     }
     TextLabel3D* residueLabel = createLabel(text, pos, radius, color);
     geometry->addDrawable(residueLabel);
@@ -392,8 +383,16 @@ void Label::processAtom(const Core::Molecule& molecule,
       text += (text == "" ? "" : " / ") + std::to_string(atom.index() + 1);
     }
     if (interface->atomOptions & LayerLabel::LabelOptions::Name) {
-      text +=
-        (text == "" ? "" : " / ") + std::string(Elements::symbol(atomicNumber));
+      std::string name = std::string(Elements::symbol(atomicNumber));
+      // D or T for hydrogen isotopes
+      if (atomicNumber == 1) {
+        if (atom.isotope() == 2) {
+          name = "D";
+        } else if (atom.isotope() == 3) {
+          name = "T";
+        }
+      }
+      text += (text == "" ? "" : " / ") + name;
     }
     if (interface->atomOptions & LayerLabel::LabelOptions::Ordinal) {
       text += (text == "" ? "" : " / ") +
@@ -405,18 +404,20 @@ void Label::processAtom(const Core::Molecule& molecule,
     }
     if (text != "") {
       const Vector3f pos(atom.position3d().cast<float>());
-      Vector3ub color = interface->color;
+      Vector3ub color = atom.color();
       float radius = static_cast<float>(Elements::radiusVDW(atomicNumber)) *
                      interface->radiusScalar;
 
-      TextLabel3D* atomLabel = createLabel(text, pos, radius, color);
+      TextLabel3D* atomLabel =
+        createLabel(text, pos, radius, contrastColor(color));
       geometry->addDrawable(atomLabel);
     }
   }
 }
 
 void Label::processBond(const Core::Molecule& molecule,
-                        Rendering::GroupNode& node, size_t layer)
+                        Rendering::GroupNode& node,
+                        [[maybe_unused]] size_t layer)
 {
   auto* geometry = new GeometryNode;
   node.addChild(geometry);
@@ -491,6 +492,9 @@ void Label::atomLabelType(int index)
                                   ->itemData(index)
                                   .toInt());
   emit drawablesChanged();
+
+  QSettings settings;
+  settings.setValue("label/atomoptions", interface->atomOptions);
 }
 
 void Label::bondLabelType(int index)
@@ -501,6 +505,9 @@ void Label::bondLabelType(int index)
                                   ->itemData(index)
                                   .toInt());
   emit drawablesChanged();
+
+  QSettings settings;
+  settings.setValue("label/bondoptions", interface->bondOptions);
 }
 
 void Label::residueLabelType(int index)
@@ -511,6 +518,9 @@ void Label::residueLabelType(int index)
                                      ->itemData(index)
                                      .toInt());
   emit drawablesChanged();
+
+  QSettings settings;
+  settings.setValue("label/residueoptions", interface->residueOptions);
 }
 
 void Label::setRadiusScalar(double radius)
@@ -520,7 +530,7 @@ void Label::setRadiusScalar(double radius)
   emit drawablesChanged();
 
   QSettings settings;
-  settings.setValue("label/radiusScalar", interface->radiusScalar);
+  settings.setValue("label/radiusscalar", interface->radiusScalar);
 }
 
 QWidget* Label::setupWidget()
