@@ -32,6 +32,7 @@
 namespace Avogadro {
 
 using QtGui::Molecule;
+using QtPlugins::MolecularModel;
 
 MolecularView::MolecularView(QWidget* parent)
   : QTableView(parent), m_molecule(nullptr), m_model(nullptr)
@@ -63,15 +64,6 @@ void MolecularView::setMolecule(Molecule* molecule)
   m_molecule = molecule;
 }
 
-void MolecularView::hideEvent(QHideEvent*)
-{
-  if (model()) {
-    model()->deleteLater();
-  }
-
-  this->deleteLater();
-}
-
 void MolecularView::keyPressEvent(QKeyEvent* event)
 {
   // handle copy event
