@@ -16,17 +16,20 @@ namespace QtGui {
 class Molecule;
 }
 
+namespace QtPlugins {
+class MolecularModel;
+}
+
 class MolecularView : public QTableView
 {
   Q_OBJECT
 public:
-  explicit MolecularView(QWidget* parent = 0);
+  explicit MolecularView(QWidget* parent = nullptr);
 
   void selectionChanged(const QItemSelection& selected,
                         const QItemSelection& previous) override;
   void setMolecule(QtGui::Molecule* molecule);
-  void setSourceModel(MolecularModel* model) { m_model = model; }
-  void hideEvent(QHideEvent* event) override;
+  void setSourceModel(QtPlugins::MolecularModel* model) { m_model = model; }
   void contextMenuEvent(QContextMenuEvent* event) override;
 
 protected:
@@ -35,7 +38,7 @@ protected:
 
 private:
   QtGui::Molecule* m_molecule;
-  MolecularModel* m_model;
+  QtPlugins::MolecularModel* m_model;
   void copySelectedRowsToClipboard();
   void openExportDialogBox();
 };
