@@ -37,16 +37,17 @@ using std::vector;
 
 using SecondaryStructure = Avogadro::Core::Residue::SecondaryStructure;
 
-// element, valence, formal charge, partial charge, x, y, z, label, color
-const int AtomColumns = 9;
+// element, valence, formal charge, partial charge, x, y, z, label, isotope,
+// color
+const int AtomColumns = 10;
 // type, atom 1, atom 2, bond order, length, label
 const int BondColumns = 6;
 // type, atom 1, atom 2, atom 3, angle
 const int AngleColumns = 5;
 // type, atom 1, atom 2, atom 3, atom 4, dihedral
 const int TorsionColumns = 6;
-// name, number, chain, secondary structure, heterogen, color
-const int ResidueColumns = 6;
+// name, number, chain, secondary structure, label, heterogen, color
+const int ResidueColumns = 7;
 // number, rmsd, energy or more depending on available properties
 const int ConformerColumns = 1;
 
@@ -236,8 +237,7 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
     if (m_type == ConformerType) {
       return toVariant(Qt::AlignRight | Qt::AlignVCenter); // energies
     } else if (m_type == AtomType) {
-      if ((index.column() == AtomDataCharge) ||
-          (index.column() == AtomDataColor))
+      if (index.column() == AtomDataColor)
         return toVariant(Qt::AlignRight | Qt::AlignVCenter);
       else
         return toVariant(Qt::AlignHCenter | Qt::AlignVCenter);
@@ -309,24 +309,47 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
           // Return the x coordinate as a double for sorting
           return m_molecule->atomPosition3d(row).x();
         else // format fixed to 4 decimals
-          return QString("%L1").arg(m_molecule->atomPosition3d(row).x(), 0, 'f',
-                                    4);
+        {
+          auto formatted =
+            QString("%L1").arg(m_molecule->atomPosition3d(row).x(), 0, 'f', 4);
+          if (m_molecule->frozenAtom(row) || m_molecule->frozenAtomAxis(row, 0))
+            formatted += "🔒";
+          return formatted;
+        }
+
       case AtomDataY:
         if (role == Qt::UserRole)
           // Return the y coordinate as a double for sorting
           return m_molecule->atomPosition3d(row).y();
         else // format fixed to 4 decimals
-          return QString("%L1").arg(m_molecule->atomPosition3d(row).y(), 0, 'f',
-                                    4);
+        {
+          auto formatted =
+            QString("%L1").arg(m_molecule->atomPosition3d(row).y(), 0, 'f', 4);
+          if (m_molecule->frozenAtom(row) || m_molecule->frozenAtomAxis(row, 1))
+            formatted += "🔒";
+          return formatted;
+        }
       case AtomDataZ:
         if (role == Qt::UserRole)
           // Return the z coordinate as a double for sorting
           return m_molecule->atomPosition3d(row).z();
         else // format fixed to 4 decimals
-          return QString("%L1").arg(m_molecule->atomPosition3d(row).z(), 0, 'f',
-                                    4);
+        {
+          auto formatted =
+            QString("%L1").arg(m_molecule->atomPosition3d(row).z(), 0, 'f', 4);
+          if (m_molecule->frozenAtom(row) || m_molecule->frozenAtomAxis(row, 2))
+            formatted += "🔒";
+          return formatted;
+        }
       case AtomDataLabel:
         return m_molecule->atomLabel(row).c_str();
+      case AtomDataIsotope: {
+        int isotope = m_molecule->isotope(row);
+        if (isotope > 0)
+          return isotope;
+        else
+          return QVariant();
+      }
       case AtomDataColor:
       default:
         return QVariant(); // nothing to show
@@ -344,6 +367,17 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
     auto bond = m_molecule->bond(row);
     auto atom1 = bond.atom1();
     auto atom2 = bond.atom2();
+    bool isConstrained = false;
+
+    // check constraints for this bond
+    for (auto& constraint : m_molecule->constraints()) {
+      if (constraint.aIndex() == atom1.index() &&
+          constraint.bIndex() == atom2.index()) {
+        isConstrained = true;
+        break;
+      }
+    }
+
     switch (column) {
       case BondDataType:
         return QString("%1-%2")
@@ -361,9 +395,13 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
         if (role == Qt::UserRole)
           // Return the bond length as a double for sorting
           return distance(atom1.position3d(), atom2.position3d());
-        else
-          return QString("%L1 Å").arg(
+        else {
+          QString formatted = QString("%L1").arg(
             distance(atom1.position3d(), atom2.position3d()), 0, 'f', 3);
+          if (isConstrained)
+            formatted += "🔒";
+          return formatted;
+        }
     }
   } else if (m_type == ResidueType) {
 
@@ -386,6 +424,8 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
         return QString(residue.chainId());
       case ResidueDataSecStructure:
         return secStructure(residue.secondaryStructure());
+      case ResidueDataLabel:
+        return m_molecule->residueLabel(row).c_str();
       case ResidueDataHeterogen:
         return QString(residue.isHeterogen() ? "X" : "");
       default:
@@ -406,6 +446,18 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
     Vector3 a2 = m_molecule->atomPosition3d(std::get<1>(angle));
     Vector3 a3 = m_molecule->atomPosition3d(std::get<2>(angle));
 
+    bool isConstrained = false;
+
+    // check constraints for this angle
+    for (auto& constraint : m_molecule->constraints()) {
+      if (constraint.aIndex() == std::get<0>(angle) &&
+          constraint.bIndex() == std::get<1>(angle) &&
+          constraint.cIndex() == std::get<2>(angle)) {
+        isConstrained = true;
+        break;
+      }
+    }
+
     switch (column) {
       case AngleDataType:
         return angleTypeString(atomNumber1, atomNumber2, atomNumber3);
@@ -419,8 +471,13 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
         if (role == Qt::UserRole)
           // Return the angle as a double for sorting
           return calculateAngle(a1, a2, a3);
-        else // format fixed to 3 decimals
-          return QString("%L1 °").arg(calculateAngle(a1, a2, a3), 0, 'f', 3);
+        else { // format fixed to 3 decimals
+          QString formatted =
+            QString("%L1").arg(calculateAngle(a1, a2, a3), 0, 'f', 3);
+          if (isConstrained)
+            formatted += "🔒";
+          return formatted;
+        }
       default:
         return QVariant();
     }
@@ -442,6 +499,19 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
     Vector3 a3 = m_molecule->atomPosition3d(std::get<2>(torsion));
     Vector3 a4 = m_molecule->atomPosition3d(std::get<3>(torsion));
 
+    bool isConstrained = false;
+
+    // check constraints for this torsion
+    for (auto& constraint : m_molecule->constraints()) {
+      if (constraint.aIndex() == std::get<0>(torsion) &&
+          constraint.bIndex() == std::get<1>(torsion) &&
+          constraint.cIndex() == std::get<2>(torsion) &&
+          constraint.dIndex() == std::get<3>(torsion)) {
+        isConstrained = true;
+        break;
+      }
+    }
+
     switch (column) {
       case TorsionDataType:
         return torsionTypeString(atomNumber1, atomNumber2, atomNumber3,
@@ -459,9 +529,13 @@ QVariant PropertyModel::data(const QModelIndex& index, int role) const
         if (role == Qt::UserRole)
           // Return the dihedral angle as a double for sorting
           return calculateDihedral(a1, a2, a3, a4);
-        else // format fixed to 3 decimals
-          return QString("%L1 °").arg(calculateDihedral(a1, a2, a3, a4), 0, 'f',
-                                      3);
+        else { // format fixed to 3 decimals
+          QString format =
+            QString("%L1").arg(calculateDihedral(a1, a2, a3, a4), 0, 'f', 3);
+          if (isConstrained)
+            format += "🔒";
+          return format;
+        }
       default:
         return QVariant();
     }
@@ -547,6 +621,8 @@ QVariant PropertyModel::headerData(int section, Qt::Orientation orientation,
           return tr("Z (Å)");
         case AtomDataLabel:
           return tr("Label");
+        case AtomDataIsotope:
+          return tr("Isotope");
         case AtomDataColor:
           return tr("Color");
       }
@@ -588,6 +664,8 @@ QVariant PropertyModel::headerData(int section, Qt::Orientation orientation,
           return tr("Secondary Structure");
         case ResidueDataHeterogen:
           return tr("Heterogen");
+        case ResidueDataLabel:
+          return tr("Label");
         case ResidueDataColor:
           return tr("Color");
       }
@@ -661,7 +739,7 @@ Qt::ItemFlags PropertyModel::flags(const QModelIndex& index) const
     if (index.column() == AtomDataElement ||
         index.column() == AtomDataFormalCharge || index.column() == AtomDataX ||
         index.column() == AtomDataY || index.column() == AtomDataZ ||
-        index.column() == AtomDataLabel)
+        index.column() == AtomDataLabel || index.column() == AtomDataIsotope)
       return editable;
     // TODO: Color
   } else if (m_type == BondType) {
@@ -669,6 +747,8 @@ Qt::ItemFlags PropertyModel::flags(const QModelIndex& index) const
         index.column() == BondDataLabel)
       return editable;
   } else if (m_type == ResidueType) {
+    if (index.column() == ResidueDataLabel)
+      return editable;
     // TODO: Color
   } else if (m_type == AngleType) {
     if (index.column() == AngleDataValue)
@@ -726,18 +806,41 @@ bool PropertyModel::setData(const QModelIndex& index, const QVariant& value,
         } // not a number
         break;
       }
-      case AtomDataX:
-        v[0] = value.toDouble();
+      case AtomDataX: {
+        bool ok;
+        double x = value.toDouble(&ok);
+        if (ok) {
+          v[0] = x;
+        }
         break;
-      case AtomDataY:
-        v[1] = value.toDouble();
+      }
+      case AtomDataY: {
+        bool ok;
+        double y = value.toDouble(&ok);
+        if (ok) {
+          v[1] = y;
+        }
         break;
-      case AtomDataZ:
-        v[2] = value.toDouble();
+      }
+      case AtomDataZ: {
+        bool ok;
+        double z = value.toDouble(&ok);
+        if (ok) {
+          v[2] = z;
+        }
         break;
+      }
       case AtomDataLabel:
         undoMolecule->setAtomLabel(index.row(), value.toString().toStdString());
         break;
+      case AtomDataIsotope: {
+        bool ok;
+        int isotope = value.toInt(&ok);
+        if (ok) {
+          m_molecule->setIsotope(index.row(), isotope);
+        }
+        break;
+      }
       default:
         return false;
     }
@@ -749,12 +852,22 @@ bool PropertyModel::setData(const QModelIndex& index, const QVariant& value,
     return true;
   } else if (m_type == BondType) {
     switch (static_cast<BondColumn>(index.column())) {
-      case BondDataOrder:
-        undoMolecule->setBondOrder(index.row(), value.toInt());
+      case BondDataOrder: {
+        bool ok;
+        int order = value.toInt(&ok);
+        if (ok && order > 0 && order <= 6) {
+          undoMolecule->setBondOrder(index.row(), value.toInt());
+        }
         break;
-      case BondDataLength:
-        setBondLength(index.row(), value.toDouble());
+      }
+      case BondDataLength: {
+        bool ok;
+        double length = value.toDouble(&ok);
+        if (ok) {
+          setBondLength(index.row(), value.toDouble());
+        }
         break;
+      }
       case BondDataLabel:
         undoMolecule->setBondLabel(index.row(), value.toString().toStdString());
         break;
@@ -765,16 +878,31 @@ bool PropertyModel::setData(const QModelIndex& index, const QVariant& value,
     emit dataChanged(index, index);
     m_molecule->emitChanged(Molecule::Bonds);
     return true;
+  } else if (m_type == ResidueType) {
+    if (index.column() == ResidueDataLabel) {
+      m_molecule->setResidueLabel(index.row(), value.toString().toStdString());
+      emit dataChanged(index, index);
+      m_molecule->emitChanged(Molecule::Residues);
+      return true;
+    }
   } else if (m_type == AngleType) {
     if (index.column() == AngleDataValue) {
-      setAngle(index.row(), value.toDouble());
+      bool ok;
+      double angle = value.toDouble(&ok);
+      if (!ok)
+        return false;
+      setAngle(index.row(), angle);
       emit dataChanged(index, index);
       m_molecule->emitChanged(Molecule::Atoms);
       return true;
     }
   } else if (m_type == TorsionType) {
     if (index.column() == TorsionDataValue) {
-      setTorsion(index.row(), value.toDouble());
+      bool ok;
+      double angle = value.toDouble(&ok);
+      if (!ok)
+        return false;
+      setTorsion(index.row(), angle);
       emit dataChanged(index, index);
       m_molecule->emitChanged(Molecule::Atoms);
       return true;
@@ -951,6 +1079,10 @@ void PropertyModel::setMolecule(QtGui::Molecule* molecule)
   if (molecule && molecule != m_molecule) {
     m_molecule = molecule;
 
+    // Initialize structure tracking for change detection
+    m_lastAtomCount = molecule->atomCount();
+    m_lastBondCount = molecule->bondCount();
+
     updateCache();
 
     connect(m_molecule, SIGNAL(changed(unsigned int)), this,
@@ -984,17 +1116,40 @@ QString PropertyModel::secStructure(unsigned int type) const
 
 void PropertyModel::updateTable(unsigned int flags)
 {
-  if (flags & Molecule::Added || flags & Molecule::Removed) {
-    // tear it down and rebuild the model
-    updateCache();
-    beginResetModel();
-    endResetModel();
-  } else {
-    // we can just update the current data
-    emit dataChanged(
-      QAbstractItemModel::createIndex(0, 0),
-      QAbstractItemModel::createIndex(rowCount(), columnCount()));
+  // During animation/vibration, coordinates change rapidly but the table
+  // structure (number of atoms, bonds, etc.) remains the same. We can skip
+  // updates when only coordinates changed (Atoms flag without Added/Removed).
+  //
+  // Note: Some code (e.g., vibrations) incorrectly uses Added flag for
+  // coordinate changes. We detect actual structural changes by checking
+  // if the counts changed.
+  bool structureChanged = false;
+
+  if (m_molecule != nullptr) {
+    // Check if the actual structure changed (not just coordinates)
+    Index currentAtomCount = m_molecule->atomCount();
+    Index currentBondCount = m_molecule->bondCount();
+
+    if (currentAtomCount != m_lastAtomCount ||
+        currentBondCount != m_lastBondCount) {
+      structureChanged = true;
+      m_lastAtomCount = currentAtomCount;
+      m_lastBondCount = currentBondCount;
+    }
   }
+
+  if (!structureChanged) {
+    // For coordinate-only changes, just invalidate the cache
+    // This avoids race conditions during rapid animation updates
+    m_validCache = false;
+    return;
+  }
+
+  // For structural changes, do a full model reset
+  // Use beginResetModel/endResetModel to ensure thread-safe updates
+  updateCache();
+  beginResetModel();
+  endResetModel();
 }
 
 void PropertyModel::updateCache() const
