@@ -73,13 +73,9 @@ PropertyView::PropertyView(PropertyType type, QWidget* parent)
   vertical->setMinimumSectionSize(30);
   vertical->setDefaultAlignment(Qt::AlignCenter);
 
-  // You can select everything (e.g., to copy, select all, etc.)
   setCornerButtonEnabled(true);
   setSelectionBehavior(QAbstractItemView::SelectRows);
-  if (type == ConformerType)
-    setSelectionMode(QAbstractItemView::SingleSelection);
-  else
-    setSelectionMode(QAbstractItemView::ExtendedSelection);
+  setSelectionMode(QAbstractItemView::SingleSelection);
   // Alternating row colors
   setAlternatingRowColors(true);
   // Allow sorting the table
@@ -87,19 +83,28 @@ PropertyView::PropertyView(PropertyType type, QWidget* parent)
 }
 
 void PropertyView::selectionChanged(const QItemSelection& selected,
-                                    const QItemSelection&)
+                                    const QItemSelection& deselected)
 {
+  // Guard against re-entrancy: modifying molecule selection triggers
+  // model updates which can cause recursive calls to selectionChanged
+  if (m_updatingSelection)
+    return;
+
   bool ok = false;
   if (m_molecule == nullptr)
     return;
 
+  m_updatingSelection = true;
+
   // Start by clearing the molecule selection
   for (Index i = 0; i < m_molecule->atomCount(); ++i)
     m_molecule->undoMolecule()->setAtomSelected(i, false);
 
   foreach (const QModelIndex& index, selected.indexes()) {
-    if (!index.isValid())
+    if (!index.isValid()) {
+      m_updatingSelection = false;
       return;
+    }
 
     // Since the user can sort
     // we need to find the original index
@@ -110,17 +115,23 @@ void PropertyView::selectionChanged(const QItemSelection& selected,
                    .last()
                    .toLong(&ok) -
                  1;
-    if (!ok)
+    if (!ok) {
+      m_updatingSelection = false;
       return;
+    }
 
     if (m_type == PropertyType::AtomType) {
-      if (static_cast<Index>(rowNum) >= m_molecule->atomCount())
+      if (static_cast<Index>(rowNum) >= m_molecule->atomCount()) {
+        m_updatingSelection = false;
         return;
+      }
 
       m_molecule->setAtomSelected(rowNum, true);
     } else if (m_type == PropertyType::BondType) {
-      if (static_cast<Index>(rowNum) >= m_molecule->bondCount())
+      if (static_cast<Index>(rowNum) >= m_molecule->bondCount()) {
+        m_updatingSelection = false;
         return;
+      }
 
       auto bondPair = m_molecule->bondPair(rowNum);
       m_molecule->undoMolecule()->setAtomSelected(bondPair.first, true);
@@ -159,6 +170,8 @@ void PropertyView::selectionChanged(const QItemSelection& selected,
   } // end loop through selected
 
   m_molecule->emitChanged(Molecule::Atoms);
+  m_updatingSelection = false;
+  QTableView::selectionChanged(selected, deselected);
 }
 
 void PropertyView::setMolecule(Molecule* molecule)
@@ -541,7 +554,7 @@ void PropertyView::contextMenuEvent(QContextMenuEvent* event)
     menu.addAction(unfreezeAtomAction);
     connect(unfreezeAtomAction, &QAction::triggered, this,
             &PropertyView::unfreezeAtom);
-  } else {
+  } else if (m_type != PropertyType::ResidueType) {
     // bond angle & torsion are similar
     QString name;
     if (m_type == PropertyType::BondType)
