@@ -122,16 +122,23 @@ void Orbitals::loadOrbitals()
 
   if (!m_dialog) {
     m_dialog = new OrbitalWidget(qobject_cast<QWidget*>(parent()), Qt::Window);
-    connect(m_dialog, SIGNAL(orbitalSelected(unsigned int)), this,
-            SLOT(renderOrbital(unsigned int)));
-    connect(m_dialog, SIGNAL(renderRequested(unsigned int, double)), this,
-            SLOT(calculateOrbitalFromWidget(unsigned int, double)));
+    connect(m_dialog,
+            SIGNAL(orbitalSelected(unsigned int, Core::BasisSet::ElectronType)),
+            this,
+            SLOT(renderOrbital(unsigned int, Core::BasisSet::ElectronType)));
+    connect(m_dialog,
+            SIGNAL(renderRequested(unsigned int, double,
+                                   Core::BasisSet::ElectronType)),
+            this,
+            SLOT(calculateOrbitalFromWidget(unsigned int, double,
+                                            Core::BasisSet::ElectronType)));
     connect(m_dialog, SIGNAL(calculateAll()), this,
             SLOT(precalculateOrbitals()));
   }
 
   m_dialog->fillTable(m_basis);
   m_dialog->show();
+  m_dialog->raise();
 }
 
 void Orbitals::moleculeChanged([[maybe_unused]] unsigned int changes)
@@ -158,10 +165,16 @@ void Orbitals::openDialog()
 {
   if (!m_dialog) {
     m_dialog = new OrbitalWidget(qobject_cast<QWidget*>(parent()), Qt::Window);
-    connect(m_dialog, SIGNAL(orbitalSelected(unsigned int)), this,
-            SLOT(renderOrbital(unsigned int)));
-    connect(m_dialog, SIGNAL(renderRequested(unsigned int, double)), this,
-            SLOT(calculateOrbitalFromWidget(unsigned int, double)));
+    connect(m_dialog,
+            SIGNAL(orbitalSelected(unsigned int, Core::BasisSet::ElectronType)),
+            this,
+            SLOT(renderOrbital(unsigned int, Core::BasisSet::ElectronType)));
+    connect(m_dialog,
+            SIGNAL(renderRequested(unsigned int, double,
+                                   Core::BasisSet::ElectronType)),
+            this,
+            SLOT(calculateOrbitalFromWidget(unsigned int, double,
+                                            Core::BasisSet::ElectronType)));
     connect(m_dialog, SIGNAL(calculateAll()), this,
             SLOT(precalculateOrbitals()));
   }
@@ -170,15 +183,17 @@ void Orbitals::openDialog()
   m_dialog->raise();
 }
 
-void Orbitals::calculateOrbitalFromWidget(unsigned int orbital,
-                                          double resolution)
+void Orbitals::calculateOrbitalFromWidget(
+  unsigned int orbital, double resolution,
+  Core::BasisSet::ElectronType electronType)
 {
   m_updateMesh = true;
 
-  // check if the orbital is already in the queue
+  // check if the orbital is already in the queue (with same electron type)
   bool found = false;
   for (int i = 0; i < m_queue.size(); i++) {
-    if (m_queue[i].orbital == orbital && m_queue[i].resolution == resolution) {
+    if (m_queue[i].orbital == orbital && m_queue[i].resolution == resolution &&
+        m_queue[i].electronType == electronType) {
       // change the priority to the highest
       m_queue[i].priority = 0;
       found = true;
@@ -187,7 +202,8 @@ void Orbitals::calculateOrbitalFromWidget(unsigned int orbital,
   }
 
   if (!found) {
-    addCalculationToQueue(orbital, resolution, m_dialog->isovalue(), 0);
+    addCalculationToQueue(orbital, resolution, m_dialog->isovalue(),
+                          electronType, 0);
   }
   checkQueue();
 }
@@ -225,6 +241,8 @@ void Orbitals::precalculateOrbitals()
     endIndex = m_basis->molecularOrbitalCount() - 1;
   }
 
+  // For now, precalculate paired/alpha orbitals only
+  // TODO: For open-shell, could precalculate both alpha and beta
   for (unsigned int i = startIndex; i <= endIndex; i++) {
 #ifndef NDEBUG
     qDebug() << " precalculate " << i << " priority " << priority;
@@ -232,7 +250,7 @@ void Orbitals::precalculateOrbitals()
     addCalculationToQueue(
       i, // orbital
       OrbitalWidget::OrbitalQualityToDouble(m_dialog->defaultQuality()),
-      m_dialog->isovalue(), priority);
+      m_dialog->isovalue(), Core::BasisSet::Paired, priority);
 
     // Update priority. Stays the same when i = homo.
     if (i + 1 < homo)
@@ -244,11 +262,14 @@ void Orbitals::precalculateOrbitals()
 }
 
 void Orbitals::addCalculationToQueue(unsigned int orbital, double resolution,
-                                     double isovalue, unsigned int priority)
+                                     double isovalue,
+                                     Core::BasisSet::ElectronType electronType,
+                                     unsigned int priority)
 {
   // Create new queue entry
   calcInfo newCalc;
   newCalc.orbital = orbital;
+  newCalc.electronType = electronType;
   newCalc.resolution = resolution;
   newCalc.isovalue = isovalue;
   newCalc.priority = priority;
@@ -256,9 +277,6 @@ void Orbitals::addCalculationToQueue(unsigned int orbital, double resolution,
 
   // Add new calculation
   m_queue.append(newCalc);
-
-  // Set progress to show 0%
-  m_dialog->calculationQueued(newCalc.orbital);
 }
 
 void Orbitals::checkQueue()
@@ -335,17 +353,25 @@ void Orbitals::calculateCube()
 
   info->state = Running;
 
-  // Check if the cube we want already exists
+  // Check if the cube we want already exists (must match electron type too)
   for (int i = 0; i < m_queue.size(); i++) {
     calcInfo* cI = &m_queue[i];
     if (cI->state == Completed && cI->orbital == info->orbital &&
-        cI->resolution == info->resolution) {
+        cI->resolution == info->resolution &&
+        cI->electronType == info->electronType) {
       info->cube = cI->cube;
 #ifndef NDEBUG
       qDebug() << "Reusing cube from calculation " << i << ":\n"
                << "\tOrbital " << cI->orbital << "\n"
                << "\tResolution " << cI->resolution;
 #endif
+      // Set this cube as the active cube for volume rendering
+      for (Index j = 0; j < m_molecule->cubeCount(); ++j) {
+        if (m_molecule->cube(j) == cI->cube) {
+          m_molecule->setActiveCubeIndex(j);
+          break;
+        }
+      }
       m_currentMeshCalculation = m_currentRunningCalculation;
       calculatePosMesh();
       calculationComplete();
@@ -360,6 +386,9 @@ void Orbitals::calculateCube()
   cube->setName("Molecular Orbital " + std::to_string(info->orbital + 1));
   cube->setCubeType(Core::Cube::Type::MO);
 
+  // Set this cube as the active cube for volume rendering
+  m_molecule->setActiveCubeIndex(m_molecule->cubeCount() - 1);
+
   if (!m_gaussianConcurrent) {
     m_gaussianConcurrent = new QtGui::GaussianSetConcurrent(this);
   }
@@ -368,17 +397,11 @@ void Orbitals::calculateCube()
   auto* watcher = &m_gaussianConcurrent->watcher();
   connect(watcher, SIGNAL(finished()), this, SLOT(calculateCubeDone()));
 
-  m_dialog->initializeProgress(info->orbital, watcher->progressMinimum(),
-                               watcher->progressMaximum(), 1, 3);
-
-  connect(watcher, SIGNAL(progressValueChanged(int)), this,
-          SLOT(updateProgress(int)));
-
 #ifndef NDEBUG
   qDebug() << info->orbital << " Cube calculation started.";
 #endif
-  // TODO: add alpha / beta
-  m_gaussianConcurrent->calculateMolecularOrbital(cube, info->orbital);
+  bool beta = (info->electronType == Core::BasisSet::Beta);
+  m_gaussianConcurrent->calculateMolecularOrbital(cube, info->orbital, beta);
 }
 
 void Orbitals::calculateCubeDone()
@@ -418,7 +441,7 @@ void Orbitals::calculatePosMesh()
 
 void Orbitals::calculatePosMeshDone()
 {
-  disconnect(m_meshGenerator, 0, this, 0);
+  disconnect(m_meshGenerator, nullptr, this, nullptr);
   calculateNegMesh();
 }
 
@@ -445,9 +468,13 @@ void Orbitals::calculateNegMesh()
 
 void Orbitals::calculateNegMeshDone()
 {
-  disconnect(m_meshGenerator, 0, this, 0);
+  disconnect(m_meshGenerator, nullptr, this, nullptr);
 
   meshComplete();
+  // make sure the display type is turned on
+  QStringList displayTypes;
+  displayTypes << tr("Surfaces");
+  emit requestActiveDisplayTypes(displayTypes);
 
   // ask for a repaint
   m_molecule->emitChanged(QtGui::Molecule::Added);
@@ -475,8 +502,6 @@ void Orbitals::calculationComplete()
 
   calcInfo* info = &m_queue[m_currentRunningCalculation];
 
-  m_dialog->calculationComplete(info->orbital);
-
   info->state = Completed;
   m_currentRunningCalculation = -1;
   m_runningCube = false;
@@ -487,27 +512,23 @@ void Orbitals::calculationComplete()
   checkQueue();
 }
 
-void Orbitals::renderOrbital(unsigned int row)
+void Orbitals::renderOrbital(unsigned int orbital,
+                             Core::BasisSet::ElectronType electronType)
 {
-  if (row == 0)
-    return;
-
-  // table rows are indexed from 1
-  // orbitals are indexed from 0
-  unsigned int orbital = row - 1;
-
 #ifndef NDEBUG
-  qDebug() << "Rendering orbital " << orbital;
+  qDebug() << "Rendering orbital " << orbital
+           << " electronType: " << static_cast<int>(electronType);
 #endif
 
-  // Find the most recent calc matching the selected orbital:
+  // Find the most recent calc matching the selected orbital and electron type:
   calcInfo calc;
   int index = -1;
   // in the event of ties, pick the best resolution
   double resolution = OrbitalWidget::OrbitalQualityToDouble(0);
   for (int i = 0; i < m_queue.size(); i++) {
     calc = m_queue[i];
-    if (calc.orbital == orbital && calc.state == Completed) {
+    if (calc.orbital == orbital && calc.electronType == electronType &&
+        calc.state == Completed) {
       if (calc.resolution <= resolution) {
         resolution = calc.resolution;
         index = i;
@@ -518,9 +539,20 @@ void Orbitals::renderOrbital(unsigned int row)
   // calculate the meshes
   if (index == -1) {
     // need to calculate the cube first
-    calculateOrbitalFromWidget(orbital, OrbitalWidget::OrbitalQualityToDouble(
-                                          m_dialog->defaultQuality()));
+    calculateOrbitalFromWidget(
+      orbital,
+      OrbitalWidget::OrbitalQualityToDouble(m_dialog->defaultQuality()),
+      electronType);
   } else {
+    // Set this cube as the active cube for volume rendering
+    Core::Cube* activeCube = m_queue[index].cube;
+    for (Index j = 0; j < m_molecule->cubeCount(); ++j) {
+      if (m_molecule->cube(j) == activeCube) {
+        m_molecule->setActiveCubeIndex(j);
+        break;
+      }
+    }
+
     // just need to update the meshes
     if (m_currentMeshCalculation == -1) {
       m_currentMeshCalculation = index;
@@ -533,18 +565,8 @@ void Orbitals::renderOrbital(unsigned int row)
 
   // add the orbital to the renderer
   QStringList displayTypes;
-  displayTypes << tr("Meshes");
-  requestActiveDisplayTypes(displayTypes);
-}
-
-void Orbitals::updateProgress(int current)
-{
-  if (m_currentRunningCalculation == -1)
-    return;
-
-  calcInfo* info = &m_queue[m_currentRunningCalculation];
-  int orbital = info->orbital;
-  m_dialog->updateProgress(orbital, current);
+  displayTypes << tr("Surfaces");
+  emit requestActiveDisplayTypes(displayTypes);
 }
 
 } // namespace Avogadro::QtPlugins
