@@ -10,6 +10,7 @@
 
 #include <avogadro/core/avogadrocore.h>
 #include <avogadro/qtgui/molecule.h>
+#include <avogadro/qtgui/pluginlayermanager.h>
 #include <avogadro/rendering/primitive.h>
 
 #include <QtCore/QPoint>
@@ -29,7 +30,7 @@ class TemplateTool : public QtGui::ToolPlugin
 {
   Q_OBJECT
 public:
-  explicit TemplateTool(QObject* parent_ = NULL);
+  explicit TemplateTool(QObject* parent_ = nullptr);
   ~TemplateTool() override;
 
   QString name() const override { return tr("Template tool"); }
@@ -100,6 +101,8 @@ private:
   bool m_fixValenceLater;
   QString m_keyPressBuffer;
 
+  QtGui::PluginLayerManager m_layerManager;
+
   Real m_bondDistance;
 };
 
