@@ -7,6 +7,7 @@
 
 #include <avogadro/core/elements.h>
 #include <avogadro/core/molecule.h>
+#include <avogadro/core/unitcell.h>
 #include <avogadro/core/utilities.h>
 
 #include <iostream>
@@ -75,11 +76,28 @@ void NWChemLog::processLine(std::istream& in, Core::Molecule& mol)
   if (Core::contains(key, "Output coordinates")) {
     if (mol.atomCount())
       mol.clearAtoms();
+
+    // get conversion factor
+    vector<string> list = Core::split(key, ' ');
+    if (list.size() == 11)
+      // conversion to Bohr
+      m_coordinateScale = Core::lexicalCast<double>(list[6]).value_or(1.0);
+
     readAtoms(in, mol);
   } else if (Core::contains(key, "P.Frequency")) {
     readFrequencies(line, in, mol);
   } else if (Core::contains(key, "Projected Infra")) {
     readIntensities(in, mol);
+  } else if (Core::contains(key, "lattice vectors") &&
+             !Core::contains(key, "reciprocal")) {
+    // check the lattic conversion factor
+    Real factor = 1.0;
+    vector<string> list = Core::split(key, ' ');
+    if (list.size() == 11)
+      // conversion to Bohr
+      factor = Core::lexicalCast<double>(list[6]).value_or(1.0);
+
+    readLattice(in, mol, factor);
   }
 }
 
@@ -91,6 +109,11 @@ void NWChemLog::readAtoms(std::istream& in, Core::Molecule& mol)
     if (!getline(in, line))
       return;
 
+  // if m_coordinateScale is 1.0 NWChem output Bohr and we need to convert
+  Real scale = 1.0;
+  if (m_coordinateScale == 1.0)
+    scale = BOHR_TO_ANGSTROM;
+
   while (true) {
     if (!getline(in, line))
       return;
@@ -98,7 +121,12 @@ void NWChemLog::readAtoms(std::istream& in, Core::Molecule& mol)
     // Keep going until the expected number of components is not seen.
     if (parts.size() != 6)
       break;
-    unsigned char element = Core::Elements::atomicNumberFromSymbol(parts[1]);
+
+    std::string symbol = parts[1];
+    // upper-case the first character
+    symbol[0] = toupper(symbol[0]);
+
+    unsigned char element = Core::Elements::atomicNumberFromSymbol(symbol);
     if (element == Avogadro::InvalidElement) {
       appendError("Invalid element encountered: " + parts[1]);
       return;
@@ -114,10 +142,53 @@ void NWChemLog::readAtoms(std::istream& in, Core::Molecule& mol)
       }
     }
     Core::Atom a = mol.addAtom(element);
-    a.setPosition3d(p);
+    a.setPosition3d(p * scale);
   }
 }
 
+void NWChemLog::readLattice(std::istream& in, Core::Molecule& mol,
+                            double factor)
+{
+  Real scale = 1.0;
+  if (factor == 1.0)
+    scale = BOHR_TO_ANGSTROM;
+
+  string line;
+  if (!getline(in, line))
+    return;
+
+  // next 3 lines should be a1, a2, a3
+  getline(in, line);
+  vector<string> parts = Core::split(line, ' ');
+  if (parts.size() != 5)
+    return;
+  Vector3 a1;
+  a1[0] = Core::lexicalCast<double>(parts[1]).value_or(0.0);
+  a1[1] = Core::lexicalCast<double>(parts[2]).value_or(0.0);
+  a1[2] = Core::lexicalCast<double>(parts[3]).value_or(0.0);
+
+  getline(in, line);
+  parts = Core::split(line, ' ');
+  if (parts.size() != 5)
+    return;
+  Vector3 a2;
+  a2[0] = Core::lexicalCast<double>(parts[1]).value_or(0.0);
+  a2[1] = Core::lexicalCast<double>(parts[2]).value_or(0.0);
+  a2[2] = Core::lexicalCast<double>(parts[3]).value_or(0.0);
+
+  getline(in, line);
+  parts = Core::split(line, ' ');
+  if (parts.size() != 5)
+    return;
+  Vector3 a3;
+  a3[0] = Core::lexicalCast<double>(parts[1]).value_or(0.0);
+  a3[1] = Core::lexicalCast<double>(parts[2]).value_or(0.0);
+  a3[2] = Core::lexicalCast<double>(parts[3]).value_or(0.0);
+
+  auto* cell = new Core::UnitCell(a1 * scale, a2 * scale, a3 * scale);
+  mol.setUnitCell(cell);
+}
+
 void NWChemLog::readFrequencies(const std::string& firstLine, std::istream& in,
                                 Core::Molecule&)
 {
