@@ -6,8 +6,12 @@
 
 HERE="$(dirname "$(readlink -f "${0}")")"
 
+export LC_NUMERIC=C
 export LD_LIBRARY_PATH="${HERE}/usr/lib:${LD_LIBRARY_PATH}"
-#export BABEL_DATADIR=$(readlink -f "${HERE}/usr/share/openbabel/*/")
-#export BABEL_LIBDIR=$(readlink -f "${HERE}/usr/lib/openbabel/")
+
+# Only force xcb if on Wayland and user hasn't set a preference
+if [ -n "$WAYLAND_DISPLAY" ] && [ -z "$QT_QPA_PLATFORM" ]; then
+    export QT_QPA_PLATFORM=xcb
+fi
 
 exec "${HERE}/usr/bin/avogadro2" "$@"
